//+------------------------------------------------------------------+
//|                                                     TSI_MACD.mq4 |
//|                      Copyright  2006, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_maximum 110
#property indicator_minimum -110
#property indicator_level1 0
//---- input parameters

enum enTimeFrames
{
   tf_cu  = 0,              // Current time frame
   tf_m1  = PERIOD_M1,      // 1 minute
   tf_m5  = PERIOD_M5,      // 5 minutes
   tf_m15 = PERIOD_M15,     // 15 minutes
   tf_m30 = PERIOD_M30,     // 30 minutes
   tf_h1  = PERIOD_H1,      // 1 hour
   tf_h4  = PERIOD_H4,      // 4 hours
   tf_d1  = PERIOD_D1,      // Daily
   tf_w1  = PERIOD_W1,      // Weekly
   tf_mn1 = PERIOD_MN1      // Monthly
};

extern enTimeFrames TimeFrame = tf_cu;   // Time frame to use
extern int Fast = 8;
extern int Slow = 21;
extern int Signal = 5;
extern ENUM_APPLIED_PRICE PriceToUse=PRICE_CLOSE;
extern int First_R = 20;
extern int Second_S = 5;
extern int SignalPeriod = 5;
extern ENUM_MA_METHOD Mode_Smooth = 2;
extern bool Interpolate=true;
//---- buffers
double TSI_Buffer[];
double SignalBuffer[];
double MTM_Buffer[];
double EMA_MTM_Buffer[];
double EMA2_MTM_Buffer[];
double ABSMTM_Buffer[];
double EMA_ABSMTM_Buffer[];
double EMA2_ABSMTM_Buffer[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   IndicatorBuffers(8);
   SetIndexBuffer(0, TSI_Buffer);   SetIndexLabel(0, "TSI_MACD");
   SetIndexBuffer(1, SignalBuffer);  SetIndexLabel(1, "Signal");
   SetIndexBuffer(2, MTM_Buffer);
   SetIndexBuffer(3, EMA_MTM_Buffer);
   SetIndexBuffer(4, EMA2_MTM_Buffer);
   SetIndexBuffer(5, ABSMTM_Buffer);
   SetIndexBuffer(6, EMA_ABSMTM_Buffer);
   SetIndexBuffer(7, EMA2_ABSMTM_Buffer);
//----
        indicatorFileName = WindowExpertName();
        returnBars        = TimeFrame == -99;
        TimeFrame         = MathMax(TimeFrame,_Period);   

   IndicatorShortName(timeFrameToString(TimeFrame)+" TSI_MACD(" + Fast + ", " + Slow + ", " + Signal + "; " + 
                      First_R + ", " + Second_S + ", " + SignalPeriod + ")");
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMin(Bars-counted_bars,Bars-1);
            if (returnBars) { TSI_Buffer[0] = limit+1; return(0); }
            if (TimeFrame!=_Period)
            {
               limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period())); 
               for(i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,TimeFrame,Time[i]);
                     TSI_Buffer[i]   = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,Fast,Slow,Signal,PriceToUse,First_R,Second_S,SignalPeriod,Mode_Smooth,0,y);
                     SignalBuffer[i] = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,Fast,Slow,Signal,PriceToUse,First_R,Second_S,SignalPeriod,Mode_Smooth,1,y);

                  //
                  //
                  //
                  //
                  //
      
                     if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;

                  //
                  //
                  //
                  //
                  //

                     int n,k; datetime time = iTime(NULL,TimeFrame,y);
                        for(n = 1; i+n<Bars && Time[i+n] >= time; n++) continue;	
                        for(k = 1; i+n<Bars && i+k<Bars && k<n; k++) {
                           TSI_Buffer[i+k]   = TSI_Buffer[i]   + (TSI_Buffer[i+n]   - TSI_Buffer[i]  )*k/n;
                           SignalBuffer[i+k] = SignalBuffer[i] + (SignalBuffer[i+n] - SignalBuffer[i])*k/n; }
               }
               return(0);
            }   

   //
   //
   //
   //
   //
   
   for(i = limit; i >= 0; i--)
     {
       MTM_Buffer[i]         = iMACD(NULL,0,Fast,Slow,Signal,PriceToUse,MODE_MAIN,i) -  iMACD(NULL,0,Fast,Slow,Signal,PriceToUse,MODE_MAIN,i+1);  
       ABSMTM_Buffer[i]      = MathAbs(MTM_Buffer[i]);
       EMA_MTM_Buffer[i]     = iEma(MTM_Buffer[i]       ,First_R ,i,0);
       EMA_ABSMTM_Buffer[i]  = iEma(ABSMTM_Buffer[i]    ,First_R ,i,1);
       EMA2_MTM_Buffer[i]    = iEma(EMA_MTM_Buffer[i]   ,Second_S,i,2);
       EMA2_ABSMTM_Buffer[i] = iEma(EMA_ABSMTM_Buffer[i],Second_S,i,3);
       if (EMA2_ABSMTM_Buffer[i]!=0)
            TSI_Buffer[i] = 100.0*EMA2_MTM_Buffer[i] / EMA2_ABSMTM_Buffer[i];
      else  TSI_Buffer[i] = 0;
   }      
   for(i = limit; i >= 0; i--) SignalBuffer[i] = iMAOnArray(TSI_Buffer, 0, SignalPeriod, 0, Mode_Smooth, i);
   return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

double workEma[][4];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r=Bars-r-1;

   workEma[r][instanceNo] = price;
   double alpha = 2.0 / (1.0+period);
      if (r>0)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

// 
//
//
//
//

string sTfTable[] = {"M1","M5","M10","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,10,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}